from typing import Dict, Optional

from tonic_textual.classes.generator_metadata.base_metadata import BaseMetadata
from tonic_textual.enums.generator_type import GeneratorType
from tonic_textual.enums.generator_version import GeneratorVersion


class PhoneNumberGeneratorMetadata(BaseMetadata):
    def __init__(
            self,
            generator_version: GeneratorVersion = GeneratorVersion.V1,
            use_us_phone_number_generator: bool = False,
            replace_invalid_numbers: bool = True,
            swaps: Optional[Dict[str,str]] = {}
    ):
        super().__init__(
                custom_generator=GeneratorType.PhoneNumber,
                generator_version=generator_version,
                swaps=swaps
        )
        self["useUsPhoneNumberGenerator"] = use_us_phone_number_generator
        self["replaceInvalidNumbers"] = replace_invalid_numbers

    @property
    def use_us_phone_number_generator(self) -> bool:
        return self["useUsPhoneNumberGenerator"]

    @use_us_phone_number_generator.setter
    def use_us_phone_number_generator(self, value: bool):
        self["useUsPhoneNumberGenerator"] = value

    @property
    def replace_invalid_numbers(self) -> bool:
        return self["replaceInvalidNumbers"]

    @replace_invalid_numbers.setter
    def replace_invalid_numbers(self, value: bool):
        self["replaceInvalidNumbers"] = value

    def to_payload(self) -> Dict:
        return dict(self)

    @staticmethod
    def from_payload(payload: Dict) -> "PhoneNumberGeneratorMetadata":
        base_metadata = BaseMetadata.from_payload(payload)

        if base_metadata.custom_generator is not GeneratorType.PhoneNumber:
            raise Exception(
                f"Invalid value for custom generator: "
                f"PhoneNumberGeneratorMetadata requires {GeneratorType.PhoneNumber.value} but got {base_metadata.custom_generator.name}"
            )

        return PhoneNumberGeneratorMetadata(
            generator_version=base_metadata.generator_version,
            use_us_phone_number_generator=payload.get("useUsPhoneNumberGenerator", False),
            replace_invalid_numbers=payload.get("replaceInvalidNumbers", True),
            swaps=base_metadata.swaps
        )

default_phone_number_generator_metadata = PhoneNumberGeneratorMetadata()


from typing import List, Dict, Optional

from tonic_textual.classes.generator_metadata.base_date_time_generator_metadata import BaseDateTimeGeneratorMetadata
from tonic_textual.classes.generator_metadata.timestamp_shift_metadata import TimestampShiftMetadata
from tonic_textual.enums.generator_type import GeneratorType
from tonic_textual.enums.generator_version import GeneratorVersion


class DateTimeGeneratorMetadata(BaseDateTimeGeneratorMetadata):
    def __init__(
            self,
            generator_version: GeneratorVersion = GeneratorVersion.V1,
            scramble_unrecognized_dates: bool = True,
            additional_date_formats: List[str] = list(),
            apply_constant_shift_to_document: bool = False,
            metadata: TimestampShiftMetadata = None,
            swaps: Optional[Dict[str,str]] = {}
    ):
        super().__init__(
            custom_generator=GeneratorType.DateTime,
            generator_version=generator_version,
            scramble_unrecognized_dates=scramble_unrecognized_dates,
            swaps=swaps
        )
        if metadata is None:
            metadata = TimestampShiftMetadata()
        self["metadata"] = metadata
        self["additionalDateFormats"] = additional_date_formats
        self["applyConstantShiftToDocument"] = apply_constant_shift_to_document

    @property
    def metadata(self) -> TimestampShiftMetadata:
        return self["metadata"]

    @metadata.setter
    def metadata(self, value: TimestampShiftMetadata):
        self["metadata"] = value

    @property
    def additional_date_formats(self) -> List[str]:
        return self["additionalDateFormats"]

    @additional_date_formats.setter
    def additional_date_formats(self, value: List[str]):
        self["additionalDateFormats"] = value

    @property
    def apply_constant_shift_to_document(self) -> bool:
        return self["applyConstantShiftToDocument"]

    @apply_constant_shift_to_document.setter
    def apply_constant_shift_to_document(self, value: bool):
        self["applyConstantShiftToDocument"] = value

    def to_payload(self) -> Dict:
        return dict(self)

    @staticmethod
    def from_payload(payload: Dict) -> "DateTimeGeneratorMetadata":
        base_metadata = BaseDateTimeGeneratorMetadata.from_payload(payload)

        if base_metadata.custom_generator is not GeneratorType.DateTime:
            raise Exception(
                f"Invalid value for custom generator: "
                f"DateTimeGeneratorMetadata requires {GeneratorType.DateTime.value} but got {base_metadata.custom_generator.name}"
            )

        metadata_payload = payload.get("metadata", {})
        ts_metadata = TimestampShiftMetadata.from_payload(metadata_payload)

        return DateTimeGeneratorMetadata(
            generator_version=base_metadata.generator_version,
            scramble_unrecognized_dates=base_metadata.scramble_unrecognized_dates,
            additional_date_formats=payload.get("additionalDateFormats", []),
            apply_constant_shift_to_document=payload.get("applyConstantShiftToDocument", False),
            metadata=ts_metadata,
            swaps=base_metadata.swaps
        )

default_date_time_generator_metadata = DateTimeGeneratorMetadata()



from typing import Dict


class AgeShiftMetadata(dict):
    def __init__(
            self,
            age_shift_in_years: int = 7
    ):
        super().__init__()
        self["_type"] = self.__class__.__name__
        self["ageShiftInYears"] = age_shift_in_years

    @property
    def age_shift_in_years(self) -> int:
        return self["ageShiftInYears"]

    @age_shift_in_years.setter
    def age_shift_in_years(self, value: int):
        self["ageShiftInYears"] = value

    def to_payload(self) -> Dict:
        return dict(self)

    @staticmethod
    def from_payload(payload: Dict) -> "AgeShiftMetadata":
        return AgeShiftMetadata(
            age_shift_in_years=payload.get("ageShiftInYears", 7)
        )

default_age_shift_metadata = AgeShiftMetadata()


from typing import Any, Dict, List

from tonic_textual.classes.common_api_responses.replacement import Replacement


class LlmGrouping(dict):
    """Represents a group of related entities"""

    def __init__(self, representative: str, entities: List[Replacement]):
        self.representative = representative
        self.entities = entities

        dict.__init__(
            self,
            representative=representative,
            entities=[e.to_dict() for e in entities]
        )

    def to_dict(self) -> Dict[str, Any]:
        return {
            "representative": self.representative,
            "entities": [e.to_dict() for e in self.entities]
        }


class GroupResponse(dict):
    """The response containing grouped entities"""

    def __init__(self, groups: List[LlmGrouping]):
        self.groups = groups

        dict.__init__(
            self,
            groups=[g.to_dict() for g in groups]
        )

    def to_dict(self) -> Dict[str, Any]:
        return {
            "groups": [g.to_dict() for g in self.groups]
        }



from typing import Dict, Optional
import warnings
from tonic_textual.classes.generator_metadata.base_metadata import BaseMetadata


class TimestampShiftMetadata(BaseMetadata):

    def __init__(
            self,
            left_shift_in_days: Optional[int] = -7,
            right_shift_in_days: Optional[int] = 7,
            time_stamp_shift_in_days: Optional[int] = None,
            swaps: Optional[Dict[str,str]] = {}):
        super().__init__(swaps=swaps)

        if time_stamp_shift_in_days is not None:
            warnings.warn("time_stamp_shift_in_days is being deprated and will not be supported past v285 of the product.")

        self["leftShiftInDays"] = left_shift_in_days
        self["rightShiftInDays"] = right_shift_in_days
        if time_stamp_shift_in_days is not None:
            self["timestampShiftInDays"] = time_stamp_shift_in_days

    @property
    def left_shift_in_days(self) -> Optional[int]:
        return self.get("leftShiftInDays")

    @left_shift_in_days.setter
    def left_shift_in_days(self, value: Optional[int]):
        self["leftShiftInDays"] = value

    @property
    def right_shift_in_days(self) -> Optional[int]:
        return self.get("rightShiftInDays")

    @right_shift_in_days.setter
    def right_shift_in_days(self, value: Optional[int]):
        self["rightShiftInDays"] = value

    @property
    def time_stamp_shift_in_days(self) -> Optional[int]:
        return self.get("timestampShiftInDays")

    @time_stamp_shift_in_days.setter
    def time_stamp_shift_in_days(self, value: Optional[int]):
        if value is not None:
            self["timestampShiftInDays"] = value
        elif "timestampShiftInDays" in self:
            del self["timestampShiftInDays"]

    def to_payload(self) -> Dict:
        return dict(self)

    @staticmethod
    def from_payload(payload: Dict) -> "TimestampShiftMetadata":
        swaps = payload.get("swaps", {})
        left_shift = payload.get("leftShiftInDays", -7)
        right_shift = payload.get("rightShiftInDays", 7)
        time_stamp_shift = payload.get("timestampShiftInDays", None)

        return TimestampShiftMetadata(
            left_shift_in_days=left_shift,
            right_shift_in_days=right_shift,
            time_stamp_shift_in_days=time_stamp_shift,
            swaps=swaps
        )

default_timestamp_shift_metadata = TimestampShiftMetadata()


from typing import Dict, Optional

from tonic_textual.classes.generator_metadata.base_metadata import BaseMetadata
from tonic_textual.enums.generator_type import GeneratorType
from tonic_textual.enums.generator_version import GeneratorVersion


class HipaaAddressGeneratorMetadata(BaseMetadata):
    def __init__(
            self,
            generator_version: GeneratorVersion = GeneratorVersion.V1,
            use_non_hipaa_address_generator: bool = False,
            replace_truncated_zeros_in_zip_code: bool = True,
            realistic_synthetic_values: bool = True,
            swaps: Optional[Dict[str,str]] = {}
    ):
        super().__init__(
            custom_generator=GeneratorType.HipaaAddressGenerator,
            generator_version=generator_version,
            swaps=swaps
        )
        self["useNonHipaaAddressGenerator"] = use_non_hipaa_address_generator
        self["replaceTruncatedZerosInZipCode"] = replace_truncated_zeros_in_zip_code
        self["realisticSyntheticValues"] = realistic_synthetic_values

    @property
    def use_non_hipaa_address_generator(self) -> bool:
        return self["useNonHipaaAddressGenerator"]

    @use_non_hipaa_address_generator.setter
    def use_non_hipaa_address_generator(self, value: bool):
        self["useNonHipaaAddressGenerator"] = value

    @property
    def replace_truncated_zeros_in_zip_code(self) -> bool:
        return self["replaceTruncatedZerosInZipCode"]

    @replace_truncated_zeros_in_zip_code.setter
    def replace_truncated_zeros_in_zip_code(self, value: bool):
        self["replaceTruncatedZerosInZipCode"] = value

    @property
    def realistic_synthetic_values(self) -> bool:
        return self["realisticSyntheticValues"]

    @realistic_synthetic_values.setter
    def realistic_synthetic_values(self, value: bool):
        self["realisticSyntheticValues"] = value

    def to_payload(self) -> Dict:
        return dict(self)

    @staticmethod
    def from_payload(payload: Dict) -> "HipaaAddressGeneratorMetadata":
        base_metadata = BaseMetadata.from_payload(payload)

        if base_metadata.custom_generator is not GeneratorType.HipaaAddressGenerator:
            raise Exception(
                f"Invalid value for custom generator: "
                f"HipaaAddressGeneratorMetadata requires {GeneratorType.HipaaAddressGenerator.value} but got {base_metadata.custom_generator}"
            )

        return HipaaAddressGeneratorMetadata(
            generator_version=base_metadata.generator_version,
            use_non_hipaa_address_generator=payload.get("useNonHipaaAddressGenerator", False),
            replace_truncated_zeros_in_zip_code=payload.get("replaceTruncatedZerosInZipCode", True),
            realistic_synthetic_values=payload.get("realisticSyntheticValues", True),
            swaps=base_metadata.swaps
        )

default_hipaa_address_generator_metadata = HipaaAddressGeneratorMetadata()


import io
import json
import os
from typing import Optional

from tonic_textual.classes.httpclient import HttpClient
from tonic_textual.classes.parse_api_responses.file_parse_result import FileParseResult


class TextualParse:
    """Wrapper class for invoking Tonic Textual API

    Parameters
    ----------
    base_url : Optional[str]
        The URL to your Tonic Textual instance. Do not include trailing backslashes. The default value is https://textual.tonic.ai.
    api_key : Optional[str]
        Optional. Your API token. Instead of providing the API token
        here, we recommended that you set the API key in your environment as the
        value of TEXTUAL_API_KEY.
    verify: bool
        Whether to verify SSL certification verification. By default, this is enabled.
    Examples
    --------
    >>> from tonic_textual.parse_api import TextualParse
    >>> textual = TonicTextualParse("https://textual.tonic.ai")
    """

    def __init__(
        self,
        base_url: str = "https://textual.tonic.ai",
        api_key: Optional[str] = None,
        verify: bool = True,
    ):
        if api_key is None:
            api_key = os.environ.get("TONIC_TEXTUAL_API_KEY")
            if api_key is None:
                raise Exception(
                    "No API key provided. Either provide an API key, or set the API "
                    "key as the value of the TEXTUAL_API_KEY environment "
                    "variable."
                )

        self.api_key = api_key
        self.client = HttpClient(base_url, self.api_key, verify)
        self.verify = verify

    def parse_file(
        self, file: io.IOBase, file_name: str, timeout: Optional[int] = None
    ) -> FileParseResult:
        """Parse a given file. To open binary files, use the 'rb' option.

        Parameters
        ----------
        file: io.IOBase
            The opened file, available for reading, to parse.
        file_name: str
            The name of the file.
        timeout: Optional[int]
            Optional timeout in seconds. Indicates to stop waiting for the parsed result after the specified time.

        Returns
        -------
        FileParseResult
            The parsed document.
        """

        files = {
            "document": (
                None,
                json.dumps({"fileName": file_name, "csvConfig": {}}),
                "application/json",
            ),
            "file": file,
        }

        response = self.client.http_post(
            "/api/parse", files=files, timeout_seconds=timeout
        )
        document = response["document"]
        file_parse_result = response["fileParseResult"]

        return FileParseResult(
            file_parse_result, self.client, document=json.loads(document)
        )

    def parse_s3_file(
        self, bucket: str, key: str, timeout: Optional[int] = None
    ) -> FileParseResult:
        """Parse a given file found in Amazon S3. Uses boto3 to fetch files from Amazon S3.

        Parameters
        ----------
        bucket: str
            The bucket that contains the file to parse.
        key: str
            The key of the file to parse.
        timeout: Optional[int]
            Optional timeout in seconds. Indicates to stop waiting for parsed result after the specified time.

        Returns
        -------
        FileParseResult
            The parsed document.
        """

        import boto3

        s3 = boto3.resource("s3")
        obj = s3.Object(bucket, key)

        file_name = key.split("/")[-1]
        return self.parse_file(obj.get()["Body"].read(), file_name, timeout=timeout)


class TonicTextualParse(TextualParse):
    pass



from typing import Dict, Optional

from tonic_textual.classes.generator_metadata.age_shift_metadata import AgeShiftMetadata
from tonic_textual.classes.generator_metadata.base_date_time_generator_metadata import BaseDateTimeGeneratorMetadata
from tonic_textual.enums.generator_type import GeneratorType
from tonic_textual.enums.generator_version import GeneratorVersion


class PersonAgeGeneratorMetadata(BaseDateTimeGeneratorMetadata):
    def __init__(
            self,
            generator_version: GeneratorVersion = GeneratorVersion.V1,
            scramble_unrecognized_dates: bool = True,
            metadata: AgeShiftMetadata = None,
            swaps: Optional[Dict[str,str]] = {}
    ):
        super().__init__(
            custom_generator=GeneratorType.PersonAge,
            generator_version=generator_version,
            scramble_unrecognized_dates=scramble_unrecognized_dates,
            swaps=swaps
        )
        if metadata is None:
            metadata = AgeShiftMetadata()
        self["metadata"] = metadata

    @property
    def metadata(self) -> AgeShiftMetadata:
        return self["metadata"]

    @metadata.setter
    def metadata(self, value: AgeShiftMetadata):
        self["metadata"] = value

    def to_payload(self) -> Dict:
        return dict(self)

    @staticmethod
    def from_payload(payload: Dict) -> "PersonAgeGeneratorMetadata":
        base_metadata = BaseDateTimeGeneratorMetadata.from_payload(payload)

        if base_metadata.custom_generator is not GeneratorType.PersonAge:
            raise Exception(
                f"Invalid value for custom generator: "
                f"PersonAgeGeneratorMetadata requires {GeneratorType.PersonAge.value} but got {base_metadata.custom_generator}"
            )

        metadata_payload = payload.get("metadata", {})
        age_metadata = AgeShiftMetadata.from_payload(metadata_payload)

        return PersonAgeGeneratorMetadata(
            generator_version=base_metadata.generator_version,
            scramble_unrecognized_dates=base_metadata.scramble_unrecognized_dates,
            metadata=age_metadata,
            swaps=base_metadata.swaps
        )

default_person_age_generator_metadata = PersonAgeGeneratorMetadata()


import json
from typing import Dict, List

from tonic_textual.classes.common_api_responses.entity_mapping import EntityMapping


class DatasetFileEntityMappings(dict):
    """The entity mappings detected for a single dataset file."""

    def __init__(
        self,
        file_id: str,
        file_name: str,
        entities: List[EntityMapping],
    ):
        self.file_id = file_id
        self.file_name = file_name
        self.entities = entities

        dict.__init__(
            self,
            file_id=file_id,
            file_name=file_name,
            entities=entities,
        )

    @classmethod
    def from_dict(cls, data: Dict) -> "DatasetFileEntityMappings":
        return cls(
            file_id=data["fileId"],
            file_name=data["fileName"],
            entities=[EntityMapping.from_dict(entity) for entity in data["entities"]],
        )

    def describe(self) -> str:
        return json.dumps(self.to_dict(), indent=2)

    def to_dict(self) -> Dict:
        return {
            "file_id": self.file_id,
            "file_name": self.file_name,
            "entities": [entity.to_dict() for entity in self.entities],
        }



import json
from typing import Dict, List

from tonic_textual.classes.common_api_responses.dataset_file_entity_mappings import (
    DatasetFileEntityMappings,
)


class DatasetEntityMappingsResponse(dict):
    """Entity mappings for a dataset, grouped by file."""

    def __init__(self, files: List[DatasetFileEntityMappings]):
        self.files = files

        dict.__init__(
            self,
            files=files,
        )

    @classmethod
    def from_dict(cls, data: Dict) -> "DatasetEntityMappingsResponse":
        return cls(
            files=[
                DatasetFileEntityMappings.from_dict(file_data)
                for file_data in data["files"]
            ]
        )

    def describe(self) -> str:
        return json.dumps(self.to_dict(), indent=2)

    def to_dict(self) -> Dict:
        return {
            "files": [file.to_dict() for file in self.files],
        }



from typing import Dict, Optional

from tonic_textual.classes.generator_metadata.base_metadata import BaseMetadata
from tonic_textual.enums.generator_type import GeneratorType
from tonic_textual.enums.generator_version import GeneratorVersion


class NumericValueGeneratorMetadata(BaseMetadata):
    def __init__(
            self,
            generator_version: GeneratorVersion = GeneratorVersion.V1,
            use_oracle_integer_pk_generator: bool = False,
            swaps: Optional[Dict[str,str]] = {}
    ):
        super().__init__(
            custom_generator=GeneratorType.NumericValue,
            generator_version=generator_version,
            swaps=swaps
        )
        self["useOracleIntegerPkGenerator"] = use_oracle_integer_pk_generator

    @property
    def use_oracle_integer_pk_generator(self) -> bool:
        return self["useOracleIntegerPkGenerator"]

    @use_oracle_integer_pk_generator.setter
    def use_oracle_integer_pk_generator(self, value: bool):
        self["useOracleIntegerPkGenerator"] = value

    def to_payload(self) -> Dict:
        return dict(self)

    @staticmethod
    def from_payload(payload: Dict) -> "NumericValueGeneratorMetadata":
        base_metadata = BaseMetadata.from_payload(payload)

        if base_metadata.custom_generator is not GeneratorType.NumericValue:
            raise Exception(
                f"Invalid value for custom generator: "
                f"NumericValueGeneratorMetadata requires {GeneratorType.NumericValue.value} but got {base_metadata.custom_generator.name}"
            )

        return NumericValueGeneratorMetadata(
            generator_version=base_metadata.generator_version,
            use_oracle_integer_pk_generator=payload.get("useOracleIntegerPkGenerator", False),
            swaps=base_metadata.swaps
        )

default_numeric_value_generator_metadata = NumericValueGeneratorMetadata()


import json
from typing import Dict, Optional


class EntityMapping(dict):
    """An entity detected in a dataset file and the values it maps to in output."""

    def __init__(
        self,
        label: str,
        text: str,
        redacted_text: Optional[str] = None,
        synthetic_text: Optional[str] = None,
        applied_generator_state: Optional[str] = None,
        output_text: Optional[str] = None,
        row_number: Optional[int] = None,
        column_index: Optional[int] = None,
        score: Optional[float] = None,
    ):
        self.label = label
        self.text = text
        self.redacted_text = redacted_text
        self.synthetic_text = synthetic_text
        self.applied_generator_state = applied_generator_state
        self.output_text = output_text
        self.row_number = row_number
        self.column_index = column_index
        self.score = score

        dict.__init__(
            self,
            label=label,
            text=text,
            **({} if redacted_text is None else {"redacted_text": redacted_text}),
            **({} if synthetic_text is None else {"synthetic_text": synthetic_text}),
            **(
                {}
                if applied_generator_state is None
                else {"applied_generator_state": applied_generator_state}
            ),
            **({} if output_text is None else {"output_text": output_text}),
            **({} if row_number is None else {"row_number": row_number}),
            **({} if column_index is None else {"column_index": column_index}),
            **({} if score is None else {"score": score}),
        )

    @classmethod
    def from_dict(cls, data: Dict) -> "EntityMapping":
        return cls(
            label=data["label"],
            text=data["text"],
            redacted_text=data.get("redactedText"),
            synthetic_text=data.get("syntheticText"),
            applied_generator_state=data.get("appliedGeneratorState"),
            output_text=data.get("outputText"),
            row_number=data.get("rowNumber"),
            column_index=data.get("columnIndex"),
            score=data.get("score"),
        )

    def describe(self) -> str:
        return json.dumps(self.to_dict(), indent=2)

    def to_dict(self) -> Dict:
        out = {
            "label": self.label,
            "text": self.text,
        }
        if self.redacted_text is not None:
            out["redacted_text"] = self.redacted_text
        if self.synthetic_text is not None:
            out["synthetic_text"] = self.synthetic_text
        if self.applied_generator_state is not None:
            out["applied_generator_state"] = self.applied_generator_state
        if self.output_text is not None:
            out["output_text"] = self.output_text
        if self.row_number is not None:
            out["row_number"] = self.row_number
        if self.column_index is not None:
            out["column_index"] = self.column_index
        if self.score is not None:
            out["score"] = self.score
        return out



from typing import Dict, Optional

from tonic_textual.classes.generator_metadata.base_metadata import BaseMetadata
from tonic_textual.enums.generator_type import GeneratorType
from tonic_textual.enums.generator_version import GeneratorVersion


class BaseDateTimeGeneratorMetadata(BaseMetadata):
    def __init__(
            self,
            custom_generator: Optional[GeneratorType] = None,
            generator_version: GeneratorVersion = GeneratorVersion.V1,
            scramble_unrecognized_dates: bool = True,
            swaps: Optional[Dict[str,str]] = {}
    ):
        super().__init__(
            custom_generator=custom_generator,
            generator_version=generator_version,
            swaps=swaps
        )
        self["scrambleUnrecognizedDates"] = scramble_unrecognized_dates

    @property
    def scramble_unrecognized_dates(self) -> bool:
        return self["scrambleUnrecognizedDates"]

    @scramble_unrecognized_dates.setter
    def scramble_unrecognized_dates(self, value: bool):
        self["scrambleUnrecognizedDates"] = value

    def to_payload(self) -> Dict:
        return dict(self)

    @staticmethod
    def from_payload(payload: Dict) -> "BaseDateTimeGeneratorMetadata":
        base_metadata = BaseMetadata.from_payload(payload)

        return BaseDateTimeGeneratorMetadata(
            custom_generator=base_metadata.custom_generator,
            generator_version=base_metadata.generator_version,
            swaps=base_metadata.swaps,
            scramble_unrecognized_dates=payload.get("scrambleUnrecognizedDates", True)
        )

default_base_date_time_generator_metadata = BaseDateTimeGeneratorMetadata()



from typing import Dict, Optional

from tonic_textual.enums.generator_type import GeneratorType
from tonic_textual.enums.generator_version import GeneratorVersion


class BaseMetadata(dict):
    def __init__(
            self,
            custom_generator: Optional[GeneratorType] = None,
            generator_version: GeneratorVersion = GeneratorVersion.V1,
            swaps: Optional[Dict[str,str]] = {}
    ):
        super().__init__()
        self["_type"] = self.__class__.__name__
        self["customGenerator"] = custom_generator
        self["generatorVersion"] = generator_version
        self["swaps"] = swaps if swaps is not None else {}

    @property
    def custom_generator(self) -> Optional[GeneratorType]:
        return self["customGenerator"]

    @custom_generator.setter
    def custom_generator(self, value: Optional[GeneratorType]):
        self["customGenerator"] = value

    @property
    def generator_version(self) -> GeneratorVersion:
        return self["generatorVersion"]

    @generator_version.setter
    def generator_version(self, value: GeneratorVersion):
        self["generatorVersion"] = value

    @property
    def swaps(self) -> Dict[str, str]:
        return self["swaps"]

    @swaps.setter
    def swaps(self, value: Dict[str, str]):
        self["swaps"] = value if value is not None else {}

    def to_payload(self) -> Dict:
        return dict(self)

    @staticmethod
    def from_payload(payload: Dict) -> "BaseMetadata":
        custom_generator = None
        custom_generator_string = payload.get("customGenerator", None)
        if custom_generator_string is not None:
            if isinstance(custom_generator_string, GeneratorType):
                custom_generator = custom_generator_string
            else:
                custom_generator = GeneratorType[custom_generator_string]

        generator_version_value = payload.get("generatorVersion", GeneratorVersion.V1)
        if isinstance(generator_version_value, GeneratorVersion):
            generator_version = generator_version_value
        else:
            generator_version = GeneratorVersion[generator_version_value]

        swaps = payload.get("swaps", {})

        return BaseMetadata(
            custom_generator=custom_generator,
            generator_version=generator_version,
            swaps=swaps
        )

default_base_metadata = BaseMetadata()


from typing import Dict, Optional

from tonic_textual.classes.generator_metadata.base_metadata import BaseMetadata
from tonic_textual.enums.generator_type import GeneratorType
from tonic_textual.enums.generator_version import GeneratorVersion


class NameGeneratorMetadata(BaseMetadata):
    def __init__(
            self,
            generator_version: GeneratorVersion = GeneratorVersion.V1,
            is_consistency_case_sensitive: bool = False,
            preserve_gender: bool = False,
            swaps: Optional[Dict[str,str]] = {}
    ):
        super().__init__(
                custom_generator=GeneratorType.Name,
                generator_version=generator_version,
                swaps=swaps
        )
        self["isConsistencyCaseSensitive"] = is_consistency_case_sensitive
        self["preserveGender"] = preserve_gender

    @property
    def is_consistency_case_sensitive(self) -> bool:
        return self["isConsistencyCaseSensitive"]

    @is_consistency_case_sensitive.setter
    def is_consistency_case_sensitive(self, value: bool):
        self["isConsistencyCaseSensitive"] = value

    @property
    def preserve_gender(self) -> bool:
        return self["preserveGender"]

    @preserve_gender.setter
    def preserve_gender(self, value: bool):
        self["preserveGender"] = value

    def to_payload(self) -> Dict:
        return dict(self)

    @staticmethod
    def from_payload(payload: Dict) -> "NameGeneratorMetadata":
        base_metadata = BaseMetadata.from_payload(payload)

        if base_metadata.custom_generator is not GeneratorType.Name:
            raise Exception(
                f"Invalid value for custom generator: "
                f"NameGeneratorMetadata requires {GeneratorType.Name.value} but got {base_metadata.custom_generator.name}"
            )

        return NameGeneratorMetadata(
            generator_version=base_metadata.generator_version,
            is_consistency_case_sensitive=payload.get("isConsistencyCaseSensitive", False),
            preserve_gender=payload.get("preserveGender", False),
            swaps=base_metadata.swaps
        )

default_name_generator_metadata = NameGeneratorMetadata()